package com.starbuds.server.api.family.common.service.admin;

import com.starbuds.server.api.family.common.pojo.admin.FamilyAdminInfoVO;
import com.starbuds.server.api.family.common.pojo.db.FamilyAdminDO;
import com.starbuds.server.common.pojo.api.PageList;

/**
 * 管理员服务
 *
 * @author Lay
 * @date 2019/12/26
 */
public interface FaAdminService {
    /**
     * 获取管理员信息
     *
     * @param adminId
     * @return
     */
    FamilyAdminInfoVO getInfo(Long adminId);

    /**
     * 判断管理员是否存在
     *
     * @param adminId 管理员id
     * @return 是否存在
     */
    boolean isAdminExist(Long adminId);

    /**
     * 创建管理员
     *
     * @param adminDO         管理员
     * @param encryptPassword 密码
     * @return 是否成功
     */
    boolean createAdmin(FamilyAdminDO adminDO, String encryptPassword);

    /**
     * 编辑管理员
     *
     * @param adminDO 管理员
     * @return 是否成功
     */
    boolean editAdmin(FamilyAdminDO adminDO);

    /**
     * 设置管理员状态
     *
     * @param adminId
     * @param adminStatus
     * @return 是否成功
     */
    boolean setAdminStatus(Long adminId, Integer adminStatus);

    /**
     * 获取管理员列表
     *
     * @param adminName   管理员姓名
     * @param adminPhone  手机号
     * @param adminStatus 状态
     * @param pageIndex   当前页码
     * @param pageSize    分页大小
     * @param familyId    家族id
     * @return result对象
     */
    PageList<FamilyAdminInfoVO> getAdminPageList(String adminName, String adminPhone, Integer adminStatus, int pageIndex, int pageSize, Long familyId);

    /**
     * 判断手机号是否存在
     *
     * @param adminPhone 手机号
     * @return 是否存在
     */
    boolean isPhoneExist(String adminPhone);

    /**
     * 判断账号是否可用
     *
     * @param adminId
     * @return
     */
    boolean isAdminEnable(Long adminId);

    /**
     * 获取家族id
     *
     * @param adminId
     * @return
     */
    Long getFamilyId(Long adminId);

    /**
     * 是否家族长
     *
     * @param adminId
     * @param familyId
     * @return
     */
    boolean isPresident(Long adminId, Long familyId);
}
