package com.starbuds.server.api.family.common.service.admin;

import com.starbuds.server.api.family.common.pojo.admin.FamilyAuditVO;
import com.starbuds.server.api.family.common.pojo.admin.FamilySkillAuditVO;
import com.starbuds.server.api.family.common.pojo.admin.FamilyUserHisVO;
import com.starbuds.server.api.family.common.pojo.admin.FamilyUserVO;
import com.starbuds.server.common.constant.user.ApplyTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.FamilyAuditDO;
import com.starbuds.server.common.pojo.db.FamilySkillAuditDO;
import com.starbuds.server.common.pojo.db.FamilySkillDO;
import com.starbuds.server.common.pojo.db.FamilyUserDO;
import com.starbuds.server.common.pojo.finance.SkillOrderVO;
import com.starbuds.server.common.pojo.user.FamilyVO;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/15 13:59
 * @Description 家族
 */
public interface FaFamilyService {

    /**
     * 获取家族申请列表
     *
     * @param familyId
     * @param applyType
     * @param userId
     * @param auditStatus
     * @param userName
     * @param pageIndex
     * @param pageSize
     * @return
     */
    PageList<FamilyAuditVO> getFamilyAuditPageList(Long familyId, Integer applyType, Long userId, Integer auditStatus, String userName, int pageIndex, int pageSize);

    /**
     * 获取技能认证列表
     *
     * @param familyId
     * @param userId
     * @param auditStatus
     * @param pageIndex
     * @param pageSize
     * @return
     */
    PageList<FamilySkillAuditVO> getFamilySkillAuditPageList(Long familyId, Long userId, Integer auditStatus, int pageIndex, int pageSize);

    /**
     * 获取家族审核
     *
     * @param auditId
     * @return
     */
    FamilyAuditDO getFamilyAudit(Long auditId);

    /**
     * 获取家族详情
     *
     * @param familyId
     * @return
     */
    FamilyVO getFamilyDetail(Long familyId);

    /**
     * 获取用户分成比例
     *
     * @param familyId
     * @return
     */
    Double getUserRatio(Long familyId);

    /**
     * 设置用户分成比例
     *
     * @param familyId
     * @param userId
     * @param ratio
     * @return
     */
    boolean setUserRatio(Long familyId, Long userId, Double ratio);

    /**
     * 获取技能认证
     *
     * @param auditId
     * @return
     */
    FamilySkillAuditDO getFamilySkillAudit(Long auditId);

    /**
     * 设置家族签约状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditMsg
     * @param auditTime
     * @param adminId
     * @return
     */
    boolean setFamilyAuditStatus(Long auditId, Integer auditStatus, String auditMsg, Long auditTime, Long adminId);

    /**
     * 设置家族签约状态
     *
     * @param familyId
     * @param userId
     * @param applyType
     * @param auditStatus
     * @param auditMsg
     * @param auditTime
     * @param adminId
     * @return
     */
    boolean setFamilyAuditStatus(Long familyId, Long userId, Integer applyType, Integer auditStatus, String auditMsg, Long auditTime, Long adminId);

    /**
     * 设置家族签约状态
     *
     * @param familyId
     * @param userIds
     * @param applyType
     * @param auditStatus
     * @param auditMsg
     * @param auditTime
     * @param adminId
     * @return
     */
    boolean setFamilyAuditsStatus(Long familyId, Set<Long> userIds, Integer applyType, Integer auditStatus, String auditMsg, Long auditTime, Long adminId);

    /**
     * 设置家族技能认证状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditMsg
     * @param adminId
     * @param auditTime
     * @return
     */
    boolean setFamilySkillAuditStatus(Long auditId, Integer auditStatus, String auditMsg, Long adminId, Long auditTime);

    /**
     * 添加或更新
     *
     * @param userDO
     * @param applyTypeEnum
     * @return
     */
    boolean addFamilyUser(FamilyUserDO userDO, ApplyTypeEnum applyTypeEnum);

    /**
     * 添加或更新
     *
     * @param userDOs
     * @param applyTypeEnum
     * @return
     */
    boolean addFamilyUsers(List<FamilyUserDO> userDOs, ApplyTypeEnum applyTypeEnum);

    /**
     * 添加技能
     *
     * @param skillDO
     * @return
     */
    boolean addSkill(FamilySkillDO skillDO);

    /**
     * 设置技能状态
     *
     * @param familyId
     * @param userId
     * @param skillStatus
     * @param updateTime
     * @return
     */
    boolean setSkillStatus(Long familyId, Long userId, Integer skillStatus, Long updateTime);

    /**
     * 设置技能状态
     *
     * @param familyId
     * @param userIds
     * @param skillStatus
     * @param updateTime
     * @return
     */
    boolean setSkillsStatus(Long familyId, Set<Long> userIds, Integer skillStatus, Long updateTime);

    /**
     * 设置技能认证状态
     *
     * @param familyId
     * @param userId
     * @param auditStatus
     * @param auditMsg
     * @param adminId
     * @param auditTime
     * @return
     */
    boolean setSkillAuditStatus(Long familyId, Long userId, Integer auditStatus, String auditMsg, Long adminId, Long auditTime);

    /**
     * 设置技能认证状态
     *
     * @param familyId
     * @param userIds
     * @param auditStatus
     * @param auditMsg
     * @param adminId
     * @param auditTime
     * @return
     */
    boolean setSkillAuditsStatus(Long familyId, Set<Long> userIds, Integer auditStatus, String auditMsg, Long adminId, Long auditTime);

    /**
     * 判断是否有家族签约待审核
     *
     * @param familyId
     * @param userId
     * @param applyType
     * @return
     */
    boolean isFamilyApplyWaitAudit(Long familyId, Long userId, Integer applyType);

    /**
     * 判断是否有家族签约待审核
     *
     * @param familyId
     * @param userIds
     * @param applyType
     * @return
     */
    Map<Long, Boolean> isFamilyApplyWaitAudits(Long familyId, Set<Long> userIds, Integer applyType);

    /**
     * 获取技能订单
     *
     * @param pageIndex
     * @param pageSize
     * @param familyId
     * @param payStatus
     * @param orderStatus
     * @return
     */
    PageList<SkillOrderVO> getSkillOrderPageList(int pageIndex, int pageSize, Long familyId, Integer payStatus, Integer orderStatus);

    /**
     * 判断技能是否认证
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @return
     */
    boolean isSkillAuth(Long familyId, Long userId, Long skillId);

    /**
     * 移除技能
     *
     * @param auditId
     * @param reason
     * @param adminId
     * @return
     */
    boolean removeSkill(Long auditId, String reason, Long adminId);

    /**
     * 获取家族用户
     *
     * @param familyId
     * @param pageIndex
     * @param pageSize
     * @param userName
     * @return
     */
    PageList<FamilyUserVO> getFamilyUserList(Long familyId, int pageIndex, int pageSize, String userName);

    /**
     * 获取用户详情
     *
     * @param userId
     * @return
     */
    FamilyUserVO getFamilyUserDetail(Long userId);

    /**
     * 获取签约记录
     *
     * @param pageIndex
     * @param pageSize
     * @param userId
     * @return
     */
    PageList<FamilyUserHisVO> getUserHisList(int pageIndex, int pageSize, Long userId);

    /**
     * 获取家族长用户id
     *
     * @param familyId
     * @return
     */
    Long getPresident(Long familyId);

    /**
     * 解约用户
     *
     * @param familyId
     * @param adminId
     * @param userIdSet
     * @param content
     * @return
     */
    boolean setTerminate(Long familyId, Long adminId, Set<Long> userIdSet, String content);

}
