package com.starbuds.server.api.family.service.admin.dao;

import com.starbuds.server.api.family.common.pojo.admin.FamilyAdminInfoVO;
import com.starbuds.server.api.family.common.pojo.db.FamilyAdminDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-01-15 14:12:01
 * @Description 家族管理员持久层
 */
@Mapper
@Repository
public interface FamilyAdminDAO {
    /**
     * 新增
     *
     * @param entity
     * @return
     */
    int insert(FamilyAdminDO entity);

    /**
     * 根据主键更新
     *
     * @param entity
     * @return
     */
    int updateById(FamilyAdminDO entity);

    /**
     * 根据主键查询
     *
     * @param adminId
     * @return
     */
    FamilyAdminDO selectById(@Param("adminId") Long adminId);

    /**
     * 根据id获取vo
     *
     * @param adminId
     * @return
     */
    FamilyAdminInfoVO selectVOById(@Param("adminId") Long adminId);


    /**
     * 查询adminId是否存在
     *
     * @param adminId
     * @return
     */
    int countById(@Param("adminId") Long adminId);

    /**
     * 更新状态
     *
     * @param adminId
     * @param adminStatus
     * @return
     */
    int updateStatus(@Param("adminId") Long adminId, @Param("adminStatus") Integer adminStatus);

    /**
     * 获取管理员列表
     *
     * @param adminName   管理员姓名
     * @param adminPhone  售价好
     * @param adminStatus 状态
     * @param pageCursor  光标
     * @param pageSize    分页大小
     * @param familyId    家族id
     * @return 管理员列表
     */
    List<FamilyAdminInfoVO> getAdminPageList(@Param("adminName") String adminName,
                                             @Param("adminPhone") String adminPhone,
                                             @Param("adminStatus") Integer adminStatus,
                                             @Param("pageCursor") int pageCursor,
                                             @Param("pageSize") int pageSize,
                                             @Param("familyId") Long familyId);

    /**
     * 获取管理员列表大小
     *
     * @param adminName   管理员姓名
     * @param adminPhone  售价好
     * @param adminStatus 状态
     * @param familyId    家族id
     * @return 列表大小
     */
    int countAdminPageList(@Param("adminName") String adminName,
                           @Param("adminPhone") String adminPhone,
                           @Param("adminStatus") Integer adminStatus,
                           @Param("familyId") Long familyId);


    /**
     * 获取管理员状态
     *
     * @param adminId
     * @return
     */
    Integer getAdminStatus(@Param("adminId") Long adminId);

    /**
     * 获取家族id
     *
     * @param adminId
     * @return
     */
    Long selectFamilyId(@Param("adminId") Long adminId);
}
