package com.starbuds.server.api.family.service.admin.dao;

import com.starbuds.server.api.family.common.pojo.admin.FamilyAuditVO;
import com.starbuds.server.common.pojo.db.FamilyAuditDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-01-15 14:12:01
 * @Description 家族审核表持久层
 */
@Mapper
@Repository
public interface FamilyAuditDAO {

    /**
     * 根据主键更新
     *
     * @param entity
     * @return
     */
    int updateById(FamilyAuditDO entity);

    /**
     * 根据主键查询
     *
     * @param auditId
     * @return
     */
    FamilyAuditDO selectById(@Param("auditId") Long auditId);

    /**
     * 查询是否存在
     *
     * @param auditId
     * @return
     */
    int countById(@Param("auditId") Long auditId);

    /**
     * 获取入会申请列表
     *
     * @param familyId
     * @param applyType
     * @param userId
     * @param auditStatus
     * @param userName
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<FamilyAuditVO> getAuditPageList(@Param("familyId") Long familyId,
                                         @Param("applyType") Integer applyType,
                                         @Param("userId") Long userId,
                                         @Param("auditStatus") Integer auditStatus,
                                         @Param("userName") String userName,
                                         @Param("pageCursor") int pageCursor,
                                         @Param("pageSize") int pageSize);


    /**
     * 获取家族签约和解约申请列表大小
     *
     * @param familyId
     * @param applyType
     * @param userId
     * @param auditStatus
     * @param userName
     * @return
     */
    int countAuditPageList(@Param("familyId") Long familyId,
                           @Param("applyType") Integer applyType,
                           @Param("userId") Long userId,
                           @Param("auditStatus") Integer auditStatus,
                           @Param("userName") String userName);

    /**
     * 获取审核状态
     *
     * @param auditId
     * @return
     */
    Integer selectAuditStatus(@Param("auditId") Long auditId);

    /**
     * 更新状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditTime
     * @param auditMsg
     * @param adminId
     * @return
     */
    int updateStatus(@Param("auditId") Long auditId, @Param("auditStatus") Integer auditStatus, @Param("auditMsg") String auditMsg, @Param("auditTime") Long auditTime, @Param("adminId") Long adminId);

    /**
     * 更新状态
     *
     * @param familyId
     * @param userId
     * @param applyType
     * @param auditStatus
     * @param auditTime
     * @param auditMsg
     * @param adminId
     * @return
     */
    int updateStatusByUserId(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("applyType") Integer applyType, @Param("auditStatus") Integer auditStatus, @Param("auditMsg") String auditMsg, @Param("auditTime") Long auditTime, @Param("adminId") Long adminId);

    /**
     * 更新状态
     *
     * @param familyId
     * @param userIds
     * @param applyType
     * @param auditStatus
     * @param auditTime
     * @param auditMsg
     * @param adminId
     * @return
     */
    int updateStatusByUserIds(@Param("familyId") Long familyId, @Param("userIds") Set<Long> userIds, @Param("applyType") Integer applyType, @Param("auditStatus") Integer auditStatus, @Param("auditMsg") String auditMsg, @Param("auditTime") Long auditTime, @Param("adminId") Long adminId);

    /**
     * 判断是否有家族签约待审核
     *
     * @param familyId
     * @param userId
     * @param applyType
     * @param auditStatus
     * @return
     */
    int countByFamilyId(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("applyType") Integer applyType, @Param("auditStatus") Integer auditStatus);

    /**
     * 获取最近一个等待处理的主键
     *
     * @param familyId
     * @param userId
     * @param applyType
     * @param auditStatus
     * @return
     */
    Long selectLastAuditId(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("applyType") Integer applyType, @Param("auditStatus") Integer auditStatus);
}
