package com.starbuds.server.api.family.service.admin.dao;

import com.starbuds.server.api.family.common.pojo.admin.FamilyUserVO;
import com.starbuds.server.common.pojo.db.FamilyUserDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-01-15 14:12:01
 * @Description 家族签约表持久层
 */
@Mapper
@Repository
public interface FamilyUserDAO {
    /**
     * 新增
     *
     * @param entity
     * @return
     */
    int insertOrUpdate(FamilyUserDO entity);

    /**
     * 查询家族用户数量
     *
     * @param familyId
     * @return
     */
    int selectUserCount(@Param("familyId") Long familyId);

    /**
     * 查询用户分成比例
     *
     * @param familyId
     * @param userId
     * @return
     */
    Double selectUserRatio(@Param("familyId") Long familyId, @Param("userId") Long userId);

    /**
     * 设置分成比例
     *
     * @param familyId
     * @param userId
     * @param ratio
     * @return
     */
    int setUserRatio(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("ratio") Double ratio);

    /**
     * 获取家族用户
     *
     * @param familyId
     * @param pageCursor
     * @param pageSize
     * @param userName
     * @return
     */
    List<FamilyUserVO> selectFamilyUserList(@Param("familyId") Long familyId, @Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("userName") String userName);

    /**
     * 获取家族用户列表
     *
     * @param familyId
     * @param userName
     * @return
     */
    int countFamilyUserList(@Param("familyId") Long familyId, @Param("userName") String userName);

    /**
     * 获取用户详情
     *
     * @param userId
     * @return
     */
    FamilyUserVO selectFamilyUserDetail(@Param("userId") Long userId);

    /**
     * 批量插入
     *
     * @param list
     * @return
     */
    int batchInsertOrUpdate(@Param("list") List<FamilyUserDO> list);
}
