package com.starbuds.server.api.family.service.admin.impl;

import com.starbuds.server.api.family.common.pojo.admin.FamilyBankcardVO;
import com.starbuds.server.api.family.common.service.admin.FaFamilyBankcardService;
import com.starbuds.server.api.family.service.admin.dao.FamilyBankcardDAO;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.FamilyBankcardDO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/3/30 15:02
 * @Description 家族银行卡业务实现
 */
@Service
public class FaFamilyBankcardServiceImpl implements FaFamilyBankcardService {

    @Resource
    FamilyBankcardDAO familyBankcardDAO;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createFamilyBankcard(FamilyBankcardDO bankcardDO) {
        return familyBankcardDAO.insert(bankcardDO) == 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean editFamilyBankcard(FamilyBankcardDO bankcardDO) {
        return familyBankcardDAO.updateById(bankcardDO) == 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean disableFamilyBankcard(Long cardId) {
        return familyBankcardDAO.updateStatus(cardId, CommonStatus.Disabled) == 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean enableFamilyBankcard(Long cardId) {
        return familyBankcardDAO.updateStatus(cardId, CommonStatus.Normal) == 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteFamilyBankcard(Long cardId) {
        return familyBankcardDAO.updateStatus(cardId, CommonStatus.Deleted) == 1;
    }

    @Override
    public boolean isFamilyBankcardExist(Long cardId) {
        return familyBankcardDAO.countById(cardId) > 0;
    }

    @Override
    public FamilyBankcardVO getFamilyBankcardDetail(Long cardId) {
        return familyBankcardDAO.getDetail(cardId);
    }

    @Override
    public PageList<FamilyBankcardVO> getFamilyBankcardPageList(Long familyId, String cardNo, Integer cardStatus, Integer cardType, int pageIndex, int pageSize) {
        PageList<FamilyBankcardVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(familyBankcardDAO.getFamilyBankcardPageList(familyId, cardNo, cardStatus, cardType, page.getPageCursor(), pageSize));
        page.setTotal(familyBankcardDAO.countFamilyBankcardPageList(familyId, cardNo, cardStatus, cardType));
        return page;
    }

    @Override
    public Integer getCardType(Long cardId) {
        return familyBankcardDAO.selectCardType(cardId);
    }
}
