package com.starbuds.server.api.family.service.admin.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.family.common.pojo.admin.FamilyAuditVO;
import com.starbuds.server.api.family.common.pojo.admin.FamilySkillAuditVO;
import com.starbuds.server.api.family.common.pojo.admin.FamilyUserHisVO;
import com.starbuds.server.api.family.common.pojo.admin.FamilyUserVO;
import com.starbuds.server.api.family.common.service.admin.FaFamilyService;
import com.starbuds.server.api.family.config.ReleaseConfig;
import com.starbuds.server.api.family.service.admin.dao.*;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.data.TaskStatusEnum;
import com.starbuds.server.common.constant.data.TaskTypeEnum;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.constant.message.push.PushActionType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.user.ApplyTypeEnum;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.finance.SkillOrderVO;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.message.push.PushMsgParam;
import com.starbuds.server.common.pojo.mq.QueueUserExitFamilyMsg;
import com.starbuds.server.common.pojo.user.FamilyVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.SkillService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.task.PushService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.SocialRelationService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/15 14:23
 * @Description 家族
 */
@Slf4j
@Service
public class FaFamilyServiceImpl implements FaFamilyService {

    @Resource
    ReleaseConfig releaseConfig;

    @Resource
    FamilyAuditDAO familyAuditDAO;

    @Resource
    FamilyUserDAO familyUserDAO;

    @Resource
    FamilyUserHisDAO familyUserHisDAO;

    @Resource
    FamilySkillAuditDAO familySkillAuditDAO;

    @Resource
    FamilySkillDAO familySkillDAO;

    @Resource
    FamilyDAO familyDAO;

    @Resource
    SkillOrderDAO skillOrderDAO;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    PushService pushService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    SkillService skillService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    /**
     * 获取家族申请列表
     *
     * @param familyId
     * @param applyType
     * @param userId
     * @param auditStatus
     * @param userName
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<FamilyAuditVO> getFamilyAuditPageList(Long familyId, Integer applyType, Long userId, Integer auditStatus, String userName, int pageIndex, int pageSize) {
        PageList<FamilyAuditVO> page = new PageList<>(pageIndex, pageSize);
        List<FamilyAuditVO> auditList = familyAuditDAO.getAuditPageList(familyId, applyType, userId, auditStatus, userName, page.getPageCursor(), pageSize);
        if (CollectionUtil.isNotEmpty(auditList)) {
            Set<Long> userIdSet = auditList.stream().map(FamilyAuditVO::getUserId).collect(Collectors.toSet());
            Map<Long, Integer> fansCounts = socialRelationService.getFansCounts(userIdSet);
            for (FamilyAuditVO vo : auditList) {
                vo.setFansCount(CollectionUtil.isEmpty(fansCounts) ? 0 : fansCounts.get(vo.getUserId()));
                // 解约通过 且时间相同 表示家族解约
                if (vo.getApplyType() != null && vo.getApplyType() == 1 && vo.getAuditStatus() != null && vo.getAuditStatus() == AuditStatusEnum.PASS.getCode()) {
                    if (vo.getApplyTime().equals(vo.getAuditTime())) {
                        vo.setApplyType(3);
                    }
                }
            }
        }
        page.setList(auditList);
        page.setTotal(familyAuditDAO.countAuditPageList(familyId, applyType, userId, auditStatus, userName));
        return page;
    }

    /**
     * 获取技能认证列表
     *
     * @param familyId
     * @param userId
     * @param auditStatus
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<FamilySkillAuditVO> getFamilySkillAuditPageList(Long familyId, Long userId, Integer auditStatus, int pageIndex, int pageSize) {
        PageList<FamilySkillAuditVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(familySkillAuditDAO.selectFamilySkillAuditPageList(familyId, userId, auditStatus, page.getPageCursor(), pageSize));
        page.setTotal(familySkillAuditDAO.countFamilySkillAuditPageList(familyId, userId, auditStatus));
        return page;
    }

    /**
     * 获取家族审核
     *
     * @param auditId
     * @return
     */
    @Override
    public FamilyAuditDO getFamilyAudit(Long auditId) {
        return familyAuditDAO.selectById(auditId);
    }

    /**
     * 获取家族详情
     *
     * @param familyId
     * @return
     */
    @Override
    public FamilyVO getFamilyDetail(Long familyId) {
        FamilyVO vo = familyDAO.selectFamilyDetail(familyId);
        if (vo != null) {
            vo.setUserCount(familyUserDAO.selectUserCount(familyId));
        }
        return vo;
    }

    /**
     * 获取用户分成比例
     *
     * @param familyId
     * @return
     */
    @Override
    public Double getUserRatio(Long familyId) {
        return familyDAO.selectUserRatio(familyId);
    }

    /**
     * 设置用户分成比例
     *
     * @param familyId
     * @param userId
     * @param ratio
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setUserRatio(Long familyId, Long userId, Double ratio) {
        return familyUserDAO.setUserRatio(familyId, userId, ratio) > 0;
    }

    /**
     * 获取技能认证
     *
     * @param auditId
     * @return
     */
    @Override
    public FamilySkillAuditDO getFamilySkillAudit(Long auditId) {
        return familySkillAuditDAO.selectById(auditId);
    }

    /**
     * 设置家族签约状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditMsg
     * @param auditTime
     * @param adminId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setFamilyAuditStatus(Long auditId, Integer auditStatus, String auditMsg, Long auditTime, Long adminId) {
        return familyAuditDAO.updateStatus(auditId, auditStatus, auditMsg, auditTime, adminId) > 0;
    }

    /**
     * 设置家族签约状态
     *
     * @param familyId
     * @param userId
     * @param applyType
     * @param auditStatus
     * @param auditMsg
     * @param auditTime
     * @param adminId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setFamilyAuditStatus(Long familyId, Long userId, Integer applyType, Integer auditStatus, String auditMsg, Long auditTime, Long adminId) {
        return familyAuditDAO.updateStatusByUserId(familyId, userId, applyType, auditStatus, auditMsg, auditTime, adminId) > 0;
    }

    /**
     * 设置家族签约状态
     *
     * @param familyId
     * @param userIds
     * @param applyType
     * @param auditStatus
     * @param auditMsg
     * @param auditTime
     * @param adminId
     * @return
     */
    @Override
    public boolean setFamilyAuditsStatus(Long familyId, Set<Long> userIds, Integer applyType, Integer auditStatus, String auditMsg, Long auditTime, Long adminId) {
        return familyAuditDAO.updateStatusByUserIds(familyId, userIds, applyType, auditStatus, auditMsg, auditTime, adminId) > 0;
    }

    /**
     * 设置家族签约状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditMsg
     * @param adminId
     * @param auditTime
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setFamilySkillAuditStatus(Long auditId, Integer auditStatus, String auditMsg, Long adminId, Long auditTime) {
        return familySkillAuditDAO.updateStatus(auditId, auditStatus, auditMsg, adminId, auditTime) > 0;
    }

    /**
     * 添加或更新
     *
     * @param userDO
     * @param applyTypeEnum
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addFamilyUser(FamilyUserDO userDO, ApplyTypeEnum applyTypeEnum) {
        if (familyUserDAO.insertOrUpdate(userDO) > 0) {
            FamilyUserHisDO userHisDO = new FamilyUserHisDO();
            BeanUtils.copyProperties(userDO, userHisDO);
            userHisDO.setHisId(identityService.getId());
            userHisDO.setType(applyTypeEnum.getCode());
            userHisDO.setCreateTime(System.currentTimeMillis());
            familyUserHisDAO.insert(userHisDO);

            if (applyTypeEnum.equals(ApplyTypeEnum.LEAVE)) {
                QueueUserExitFamilyMsg msg = new QueueUserExitFamilyMsg();
                msg.setFamilyId(userDO.getFamilyId());
                msg.setUserId(userDO.getUserId());
                familyService.sendUserExitFamilyMsg(msg);
            }
            return true;
        }
        return false;
    }

    /**
     * 添加或更新
     *
     * @param userDOs
     * @param applyTypeEnum
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addFamilyUsers(List<FamilyUserDO> userDOs, ApplyTypeEnum applyTypeEnum) {
        if (CollectionUtil.isEmpty(userDOs)) {
            return false;
        }
        if (familyUserDAO.batchInsertOrUpdate(userDOs) > 0) {
            long curTime = System.currentTimeMillis();
            List<FamilyUserHisDO> hisList = new ArrayList<>();
            Long[] ids = identityService.getIds(userDOs.size());
            for (int i = 0; i < userDOs.size(); i++) {
                FamilyUserHisDO hisDO = new FamilyUserHisDO();
                BeanUtils.copyProperties(userDOs.get(i), hisDO);
                hisDO.setHisId(ids[i]);
                hisDO.setType(applyTypeEnum.getCode());
                hisDO.setCreateTime(curTime);
                hisList.add(hisDO);
            }
            return familyUserHisDAO.batchInsert(hisList) > 0;
        }
        return false;
    }

    /**
     * 添加技能
     *
     * @param skillDO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addSkill(FamilySkillDO skillDO) {
        if (familySkillDAO.insertOrUpdate(skillDO) > 0) {
            try {
                // 系统消息
                SystemMsg systemMsg = new SystemMsg();
                systemMsg.setContentType(ContentType.TEXT);
                systemMsg.setTitle("技能认证通过");
                String body = String.format("恭喜你，【%s】技能认证已通过，赶快去开启你的赚钱之旅吧！", skillService.getSkillName(skillDO.getSkillId()));
                systemMsg.setBody(body);
                systemMsg.setActionType(ImActionType.JUMP_NATIVE.getValue());
                systemMsg.setModuleType(ModuleTypeEnum.SKILL_AUTH.getDesc());
                systemMsg.setModuleId(Convert.toStr(0)); // 已认证
                imService.sendCustomImMsg(null, skillDO.getUserId(), systemMsg, null);
            } catch (Exception e) {
                log.error("出现错误", e);
            }
            return true;
        }
        return false;
    }

    /**
     * 设置技能状态
     *
     * @param familyId
     * @param userId
     * @param skillStatus
     * @param updateTime
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setSkillStatus(Long familyId, Long userId, Integer skillStatus, Long updateTime) {
        return familySkillDAO.updateSkillStatus(familyId, userId, skillStatus, updateTime) > 0;
    }

    /**
     * 设置技能状态
     *
     * @param familyId
     * @param userIds
     * @param skillStatus
     * @param updateTime
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setSkillsStatus(Long familyId, Set<Long> userIds, Integer skillStatus, Long updateTime) {
        return familySkillDAO.updateSkillsStatus(familyId, userIds, skillStatus, updateTime) > 0;
    }

    /**
     * 设置技能认证状态
     *
     * @param familyId
     * @param userId
     * @param auditStatus
     * @param auditMsg
     * @param adminId
     * @param auditTime
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setSkillAuditStatus(Long familyId, Long userId, Integer auditStatus, String auditMsg, Long adminId, Long auditTime) {
        return familySkillAuditDAO.updateStatusByUserId(familyId, userId, auditStatus, auditMsg, adminId, auditTime) > 0;
    }

    /**
     * 设置技能认证状态
     *
     * @param familyId
     * @param userIds
     * @param auditStatus
     * @param auditMsg
     * @param adminId
     * @param auditTime
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setSkillAuditsStatus(Long familyId, Set<Long> userIds, Integer auditStatus, String auditMsg, Long adminId, Long auditTime) {
        return familySkillAuditDAO.updateStatusByUserIds(familyId, userIds, auditStatus, auditMsg, adminId, auditTime) > 0;
    }

    /**
     * 判断是否有家族签约待审核
     *
     * @param familyId
     * @param userId
     * @param applyType
     * @return
     */
    @Override
    public boolean isFamilyApplyWaitAudit(Long familyId, Long userId, Integer applyType) {
        return familyAuditDAO.countByFamilyId(familyId, userId, applyType, AuditStatusEnum.WAIT.getCode()) > 0;
    }

    /**
     * 判断是否有家族签约待审核
     *
     * @param familyId
     * @param userIds
     * @param applyType
     * @return
     */
    @Override
    public Map<Long, Boolean> isFamilyApplyWaitAudits(Long familyId, Set<Long> userIds, Integer applyType) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Boolean> resultMap = new HashMap<>();
        for (Long userId : userIds) {
            resultMap.put(userId, familyAuditDAO.countByFamilyId(familyId, userId, applyType, AuditStatusEnum.WAIT.getCode()) > 0);
        }
        return resultMap;
    }

    /**
     * 获取技能订单
     *
     * @param pageIndex
     * @param pageSize
     * @param familyId
     * @param payStatus
     * @param orderStatus
     * @return
     */
    @Override
    public PageList<SkillOrderVO> getSkillOrderPageList(int pageIndex, int pageSize, Long familyId, Integer payStatus, Integer orderStatus) {
        PageList<SkillOrderVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(skillOrderDAO.selectSkillOrderPageList(page.getPageCursor(), pageSize, familyId, payStatus, orderStatus));
        page.setTotal(skillOrderDAO.countSkillOrderPageList(familyId, payStatus, orderStatus));
        return page;
    }

    /**
     * 判断技能是否认证
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @return
     */
    @Override
    public boolean isSkillAuth(Long familyId, Long userId, Long skillId) {
        return familySkillDAO.countById(familyId, userId, skillId) > 0;
    }

    /**
     * 移除技能
     *
     * @param auditId
     * @param reason
     * @param adminId
     * @return
     */
    @Override
    public boolean removeSkill(Long auditId, String reason, Long adminId) {
        FamilySkillAuditDO skillAudit = familySkillAuditDAO.selectById(auditId);
        if (skillAudit == null) {
            return true;
        }

        Long skillId = skillAudit.getSkillId();
        Long userId = skillAudit.getUserId();
        Long familyId = skillAudit.getFamilyId();
        long curTime = System.currentTimeMillis();

        // 作废技能
        if (familySkillDAO.updateSkill(familyId, userId, skillId, CommonStatus.Deleted, curTime) > 0) {
            // 拒绝审核记录
            familySkillAuditDAO.updateStatus(auditId, AuditStatusEnum.DENY.getCode(), reason, adminId, curTime);
            // 推送通知
            PushTaskDO taskDO = new PushTaskDO();
            taskDO.setTaskId(identityService.getId());
            taskDO.setTaskName("系统消息-技能作废");
            taskDO.setTaskStatus(TaskStatusEnum.WAIT.getValue());
            taskDO.setTaskCreateTime(System.currentTimeMillis());
            taskDO.setTaskType(TaskTypeEnum.BATCH_PUSH.getValue());
            taskDO.setTargetValue(userId.toString());
            taskDO.setAdminId(0L);
            taskDO.setRemark("系统自动推送");
            taskDO.setPushTitle("技能作废");
            String pushBody = String.format("您的技能【%s】，因为%s等原因，被%s家族驳回，已不能使用，您之前的接单记录以及收益不会被影响；如果想要再使用此技能，请重新认证此技能；", skillService.getSkillName(skillId), reason, familyDAO.selectFamilyName(familyId));
            taskDO.setPushBody(pushBody);

            PushMsgParam pushParam = new PushMsgParam();
            pushParam.setActionType(PushActionType.NO_ACTION.getValue());
            pushParam.setModuleId(userId.toString());
            taskDO.setExtParameters(JSON.toJSONString(pushParam));
            pushService.addPushTask(taskDO);

            // 系统消息
            SystemMsg systemMsg = new SystemMsg();
            systemMsg.setContentType(ContentType.TEXT);
            systemMsg.setTitle("技能被移除");
            systemMsg.setBody(String.format("由于%s%s%s，你的【%s】技能认证被移除", '"', reason, '"', skillService.getSkillName(skillId)));
            systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
            imService.sendCustomImMsg(null, userId, systemMsg, null);

        }
        return true;
    }

    /**
     * 获取家族用户
     *
     * @param familyId
     * @param pageIndex
     * @param pageSize
     * @param userName
     * @return
     */
    @Override
    public PageList<FamilyUserVO> getFamilyUserList(Long familyId, int pageIndex, int pageSize, String userName) {
        PageList<FamilyUserVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(familyUserDAO.selectFamilyUserList(familyId, page.getPageCursor(), pageSize, userName));
        page.setTotal(familyUserDAO.countFamilyUserList(familyId, userName));
        return page;
    }

    /**
     * 获取用户详情
     *
     * @param userId
     * @return
     */
    @Override
    public FamilyUserVO getFamilyUserDetail(Long userId) {
        return familyUserDAO.selectFamilyUserDetail(userId);
    }

    /**
     * 获取签约记录
     *
     * @param pageIndex
     * @param pageSize
     * @param userId
     * @return
     */
    @Override
    public PageList<FamilyUserHisVO> getUserHisList(int pageIndex, int pageSize, Long userId) {
        PageList<FamilyUserHisVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(familyUserHisDAO.selectUserHisList(page.getPageCursor(), pageSize, userId));
        page.setTotal(familyUserHisDAO.countUserHisList(userId));
        return page;
    }

    /**
     * 获取用户id
     *
     * @param familyId
     * @return
     */
    @Override
    public Long getPresident(Long familyId) {
        return familyDAO.selectPresident(familyId);
    }

    /**
     * 解约用户
     *
     * @param familyId
     * @param userIdSet
     * @param content
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setTerminate(Long familyId, Long adminId, Set<Long> userIdSet, String content) {
        long curTime = System.currentTimeMillis();
        // 家族用户
        List<FamilyUserDO> familyUsers = new ArrayList<>();
        // 家族审核记录
        List<FamilyAuditDO> auditDOS = new ArrayList<>();
        // 用户id=>是否待审核
        Map<Long, Boolean> auditMap = isFamilyApplyWaitAudits(familyId, userIdSet, ApplyTypeEnum.LEAVE.getCode());
        // 待审核的用户
        List<Long> auditExistUserIds = new ArrayList<>();
        for (Map.Entry<Long, Boolean> entry : auditMap.entrySet()) {
            if (entry.getValue()) {
                auditExistUserIds.add(entry.getKey());
            }
        }
        // 已经存在，通过审核状态
        if (CollectionUtil.isNotEmpty(auditExistUserIds)) {
            if (!setFamilyAuditsStatus(familyId, new HashSet<>(auditExistUserIds), ApplyTypeEnum.LEAVE.getCode(), AuditStatusEnum.PASS.getCode(), content, curTime, adminId)) {
                throw new RuntimeException("更新失败");
            }
        }
        Long[] auditIds = identityService.getIds(userIdSet.size() - auditExistUserIds.size());
        int i = 0;
        for (Long userId : userIdSet) {
            Boolean isExist = Optional.ofNullable(auditMap.get(userId)).orElse(false);
            // 如果不存在
            if (!isExist) {
                // 插入审核记录
                FamilyAuditDO auditDO = new FamilyAuditDO();
                auditDO.setAuditId(auditIds[i]);
                auditDO.setFamilyId(familyId);
                auditDO.setUserId(userId);
                auditDO.setApplyMsg(content);
                auditDO.setApplyTime(curTime);
                auditDO.setAuditTime(curTime);
                auditDO.setAuditStatus(AuditStatusEnum.PASS.getCode());
                auditDO.setApplyType(ApplyTypeEnum.LEAVE.getCode());
                auditDO.setIsForce(BoolIntVal.False);
                auditDO.setAdminId(adminId);
                auditDOS.add(auditDO);
                i++;
            }
            // 用户签约状态
            FamilyUserDO userDO = new FamilyUserDO();
            userDO.setUserId(userId);
            userDO.setFamilyId(familyId);
            userDO.setSignTime(null);
            userDO.setEndTime(null);
            userDO.setStatus(CommonStatus.Deleted);
            userDO.setRatio(null);
            userDO.setConnectTimes(0);
            userDO.setConnectRate(0d);
            userDO.setMatchTimes(0);
            familyUsers.add(userDO);
        }

        // 作废技能
        setSkillsStatus(familyId, userIdSet, CommonStatus.Deleted, curTime);
        setSkillAuditsStatus(familyId, userIdSet, AuditStatusEnum.DENY.getCode(), "解约家族", adminId, curTime);

        // 更新用户签约状态
        if (!addFamilyUsers(familyUsers, ApplyTypeEnum.LEAVE)) {
            throw new RuntimeException("更新失败");
        }

        // 添加审核记录
        if (auditDOS.size() > 0) {
            familyService.addFamilyAudits(auditDOS);
        }

        String pushBody = String.format("【%s】尊敬的用户您好，由于%s%s%s的原因，%s家族已与你解除家族合约。", releaseConfig.getAppName(), '"', content, '"', familyDAO.selectFamilyName(familyId));
        for (Long userId : userIdSet) {
            // 发送解约通知
            SystemMsg systemMsg = new SystemMsg();
            systemMsg.setContentType(ContentType.TEXT);
            systemMsg.setTitle("家族解约通知");
            systemMsg.setBody(pushBody);
            systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
            imService.sendCustomImMsg(null, userId, systemMsg, null);

            QueueUserExitFamilyMsg msg = new QueueUserExitFamilyMsg();
            msg.setFamilyId(familyId);
            msg.setUserId(userId);
            familyService.sendUserExitFamilyMsg(msg);
        }
        return true;
    }
}
