package com.starbuds.server.api.family.service.admin.impl;

import com.starbuds.server.api.family.common.pojo.admin.RtcRoomVO;
import com.starbuds.server.api.family.common.service.admin.FaRtcRoomService;
import com.starbuds.server.api.family.service.admin.dao.DataRtcRoomBackgroundDAO;
import com.starbuds.server.api.family.service.admin.dao.RtcRoomCategoryDAO;
import com.starbuds.server.api.family.service.admin.dao.RtcRoomDAO;
import com.starbuds.server.api.family.service.admin.dao.RtcRoomRoleDAO;
import com.starbuds.server.common.pojo.api.PageList;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/11/22 19:10
 * @Description 语音房
 */
@Service
public class FaRtcRoomServiceImpl implements FaRtcRoomService {

    @Resource
    RtcRoomDAO rtcRoomDAO;

    @Resource
    RtcRoomCategoryDAO rtcRoomCategoryDAO;

    @Resource
    RtcRoomRoleDAO rtcRoomRoleDAO;

    @Resource
    DataRtcRoomBackgroundDAO dataRtcRoomBackgroundDAO;

    /**
     * 获取列表
     *
     * @param familyId
     * @param pageIndex
     * @param pageSize
     * @param userId
     * @param roomType
     * @param roomName
     * @param categoryId
     * @param roomTitle
     * @param status
     * @return
     */
    @Override
    public PageList<RtcRoomVO> getRtcRoomPageList(Long familyId, int pageIndex, int pageSize, Long userId, Integer roomType, String roomName, Integer categoryId, String roomTitle, Integer status, Integer businessType) {
        PageList<RtcRoomVO> page = new PageList<>(pageIndex, pageSize);
        List<RtcRoomVO> list = rtcRoomDAO.selectRtcRoomPageList(familyId, page.getPageCursor(), pageSize, userId, roomType, roomName, categoryId, roomTitle, status, businessType);
        if (!CollectionUtils.isEmpty(list)) {
            for (RtcRoomVO vo : list) {
                vo.setCategoryName(rtcRoomCategoryDAO.selectCategoryName(vo.getCategoryId()));
                vo.setBackgroundUrl(dataRtcRoomBackgroundDAO.selectBackgroundUrl(vo.getBackgroundId()));
            }
        }
        page.setList(list);
        page.setTotal(rtcRoomDAO.countRtcRoomPageList(familyId, userId, roomType, roomName, categoryId, roomTitle, status, businessType));
        return page;
    }

    /**
     * 获取详情
     *
     * @param roomId
     * @return
     */
    @Override
    public RtcRoomVO getRtcRoomDetail(Long roomId) {
        RtcRoomVO roomVO = rtcRoomDAO.selectRtcRoomDetail(roomId);
        if (roomVO != null) {
            roomVO.setRoomRoleList(rtcRoomRoleDAO.selectByRoomId(roomVO.getRoomId()));
            roomVO.setCategoryName(rtcRoomCategoryDAO.selectCategoryName(roomVO.getCategoryId()));
            roomVO.setBackgroundUrl(dataRtcRoomBackgroundDAO.selectBackgroundUrl(roomVO.getBackgroundId()));
        }
        return roomVO;
    }

    /**
     * 是否有权限
     *
     * @param familyId
     * @param roomId
     */
    @Override
    public boolean hasPermission(Long familyId, Long roomId) {
        if (familyId == null) {
            return false;
        }
        return familyId.equals(rtcRoomDAO.selectFamilyId(roomId));
    }
}
