package com.starbuds.server.api.family.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/15 19:06
 * @Description 家族接口
 */
public interface FamilyFacade {

    /**
     * 获取家族申请列表
     *
     * @param applyType
     * @param userId
     * @param auditStatus
     * @param userName
     * @param pageIndex
     * @param pageSize
     * @return
     */
    Result getFamilyAuditPageList(Integer applyType, Long userId, Integer auditStatus, String userName, int pageIndex, int pageSize);

    /**
     * 获取家族技能认证申请
     *
     * @param userId
     * @param auditStatus
     * @param pageIndex
     * @param pageSize
     * @return
     */
    Result getFamilySkillAuditPageList(Long userId, Integer auditStatus, int pageIndex, int pageSize);

    /**
     * 设置家族签约状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditMsg
     * @param userRatio
     * @return
     */
    Result setFamilyAuditStatus(Long auditId, Integer auditStatus, String auditMsg, Double userRatio);

    /**
     * 设置家族技能认证状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditMsg
     * @return
     */
    Result setFamilySkillAuditStatus(Long auditId, Integer auditStatus, String auditMsg);

    /**
     * 获取家族详情
     *
     * @return
     */
    Result getFamilyDetail();

    /**
     * 获取家族默认用户分成比例
     *
     * @return
     */
    Result getUserRatio();

    /**
     * 设置家族用户分成比例
     *
     * @param userId
     * @param ratio
     * @return
     */
    Result setUserRatio(Long userId, Double ratio);

    /**
     * 获取技能订单列表
     *
     * @param pageIndex
     * @param pageSize
     * @param payStatus
     * @param orderStatus
     * @return
     */
    Result getSkillOrderPageList(int pageIndex, int pageSize, Integer payStatus, Integer orderStatus);

    /**
     * 移除技能
     *
     * @param auditId
     * @param reason
     * @return
     */
    Result removeSkill(Long auditId, String reason);

    /**
     * 获取家族成员
     *
     * @param pageIndex
     * @param pageSize
     * @param userName
     * @return
     */
    Result getFamilyUserList(int pageIndex, int pageSize, String userName);

    /**
     * 解约用户（批量）
     *
     * @param userIds
     * @param content
     * @return
     */
    Result setTerminate(String userIds, String content);

    /**
     * 获取签约记录
     *
     * @param pageIndex
     * @param pageSize
     * @param userId
     * @return
     */
    Result getUserHisList(int pageIndex, int pageSize, Long userId);

}
