package com.starbuds.server.api.family.v1.facade;

import com.starbuds.server.api.family.common.pojo.admin.FamilyBankcardReq;
import com.starbuds.server.common.pojo.api.Result;

import java.math.BigDecimal;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/10/27 13:57
 * @Description 财务接口
 */
public interface FinanceFacade {

    /**
     * 提现账户-创建卡号
     *
     * @param bankcardReq
     * @return
     */
    Result createFamilyBankcard(FamilyBankcardReq bankcardReq);

    /**
     * 提现账户-编辑卡号
     *
     * @param bankcardReq
     * @return
     */
    Result editFamilyBankcard(FamilyBankcardReq bankcardReq);

    /**
     * 提现账户-禁用卡号
     *
     * @param cardId
     * @return
     */
    Result disableFamilyBankcard(Long cardId);

    /**
     * 提现账户-启用卡号
     *
     * @param cardId
     * @return
     */
    Result enableFamilyBankcard(Long cardId);

    /**
     * 提现账户-删除卡号
     *
     * @param cardId
     * @return
     */
    Result deleteFamilyBankcard(Long cardId);

    /**
     * 提现账户-获取详情
     *
     * @param cardId
     * @return
     */
    Result getFamilyBankcardDetail(Long cardId);

    /**
     * 提现账户-获取列表
     *
     * @param cardNo
     * @param cardStatus
     * @param cardType
     * @param pageIndex
     * @param pageSize
     * @return
     */
    Result getFamilyBankcardPageList(String cardNo, Integer cardStatus, Integer cardType, int pageIndex, int pageSize);

    /**
     * 申请提现
     *
     * @param amount
     * @param cardId
     * @return
     */
    Result applyWithdraw(BigDecimal amount, Long cardId);

    /**
     * 获取余额
     *
     * @return
     */
    Result getBalance();

    /**
     * 获取家族提现申请列表
     *
     * @param applyStatus
     * @param startTime
     * @param endTime
     * @param pageIndex
     * @param pageSize
     * @return
     */
    Result getFamilyWithdrawHisPageList(Integer applyStatus, Long startTime, Long endTime, int pageIndex, int pageSize);

    /**
     * 获取账户明细列表
     *
     * @param lastId
     * @param isIncome
     * @param beginTime
     * @param endTime
     * @return
     */
    Result getWalletHis(Long lastId, int isIncome, Long beginTime, Long endTime);

    /**
     * 求和
     *
     * @param isIncome
     * @param beginTime
     * @param endTime
     * @return
     */
    Result getSumByTypeAndTime(int isIncome, Long beginTime, Long endTime);

}
