package com.starbuds.server.api.family.v1.impl;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.family.common.pojo.admin.FamilyBankcardReq;
import com.starbuds.server.api.family.common.pojo.constant.GlobalConfig;
import com.starbuds.server.api.family.common.service.admin.FaFamilyBankcardService;
import com.starbuds.server.api.family.common.service.admin.FaFamilyWalletService;
import com.starbuds.server.api.family.common.service.admin.FaFamilyWalletWithdrawHisService;
import com.starbuds.server.api.family.v1.facade.FinanceFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.CardTypeEnum;
import com.starbuds.server.common.constant.finance.WalletWithdrawHisApplyStatus;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.FamilyBankcardDO;
import com.starbuds.server.common.pojo.db.FamilyWalletWithdrawHisDO;
import com.starbuds.server.common.service.core.IdentityService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ws.rs.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.util.Arrays;
import java.util.Optional;

/**
 * 财务接口
 *
 * @author jiangxiaowei
 * @date 2020/10/27 13:58
 */
@Path("/v1/finance")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class FinanceFacadeImpl extends BaseFacade implements FinanceFacade {

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    FaFamilyBankcardService faFamilyBankcardService;

    @Resource
    FaFamilyWalletService faFamilyWalletService;

    @Resource
    FaFamilyWalletWithdrawHisService faFamilyWalletWithdrawHisService;

    @Value("${withdrawDaysOfWeek}")
    private int[] withdrawDaysOfWeek;

    @PostConstruct
    private void init() {
        log.info("可以提现的星期 {}", withdrawDaysOfWeek);
    }

    /**
     * 提现账户-创建卡号
     *
     * @param bankcardReq
     * @return
     */
    @POST
    @Path("createFamilyBankcard")
    @Override
    public Result createFamilyBankcard(@BeanParam FamilyBankcardReq bankcardReq) {
        FamilyBankcardDO bankcardDO = new FamilyBankcardDO();
        BeanUtils.copyProperties(bankcardReq, bankcardDO);
        bankcardDO.setCardId(identityService.getId());
        bankcardDO.setCardCreateTime(System.currentTimeMillis());
        bankcardDO.setFamilyId(this.getToken().getFamilyId());

        if (faFamilyBankcardService.createFamilyBankcard(bankcardDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提现账户-编辑卡号
     *
     * @param bankcardReq
     * @return
     */
    @POST
    @Path("editFamilyBankcard")
    @Override
    public Result editFamilyBankcard(@BeanParam FamilyBankcardReq bankcardReq) {
        if (!faFamilyBankcardService.isFamilyBankcardExist(bankcardReq.getCardId())) {
            return new Result(GlobalApiError.OperationFailed, "卡号不存在");
        }
        FamilyBankcardDO bankcardDO = new FamilyBankcardDO();
        BeanUtils.copyProperties(bankcardReq, bankcardDO);
        if (faFamilyBankcardService.editFamilyBankcard(bankcardDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提现账户-禁用卡号
     *
     * @param cardId
     * @return
     */
    @POST
    @Path("disableFamilyBankcard")
    @Override
    public Result disableFamilyBankcard(@FormParam("cardId") Long cardId) {
        if (!faFamilyBankcardService.isFamilyBankcardExist(cardId)) {
            return new Result(GlobalApiError.OperationFailed, "卡号不存在");
        }
        if (faFamilyBankcardService.disableFamilyBankcard(cardId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提现账户-启用卡号
     *
     * @param cardId
     * @return
     */
    @POST
    @Path("enableFamilyBankcard")
    @Override
    public Result enableFamilyBankcard(@FormParam("cardId") Long cardId) {
        if (!faFamilyBankcardService.isFamilyBankcardExist(cardId)) {
            return new Result(GlobalApiError.OperationFailed, "卡号不存在");
        }
        if (faFamilyBankcardService.enableFamilyBankcard(cardId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提现账户-删除卡号
     *
     * @param cardId
     * @return
     */
    @POST
    @Path("deleteFamilyBankcard")
    @Override
    public Result deleteFamilyBankcard(@FormParam("cardId") Long cardId) {
        if (!faFamilyBankcardService.isFamilyBankcardExist(cardId)) {
            return new Result(GlobalApiError.OperationFailed, "卡号不存在");
        }
        if (faFamilyBankcardService.deleteFamilyBankcard(cardId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提现账户-获取详情
     *
     * @param cardId
     * @return
     */
    @GET
    @Path("getFamilyBankcardDetail")
    @Override
    public Result getFamilyBankcardDetail(@QueryParam("cardId") Long cardId) {
        return new Result(faFamilyBankcardService.getFamilyBankcardDetail(cardId));
    }

    /**
     * 提现账户-获取列表
     *
     * @param cardNo
     * @param cardStatus
     * @param cardType
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @Path("getFamilyBankcardPageList")
    @Override
    public Result getFamilyBankcardPageList(@QueryParam("cardNo") String cardNo,
                                            @QueryParam("cardStatus") Integer cardStatus,
                                            @QueryParam("cardType") Integer cardType,
                                            @QueryParam("pageIndex") int pageIndex,
                                            @QueryParam("pageSize") int pageSize) {
        return new Result(faFamilyBankcardService.getFamilyBankcardPageList(this.getToken().getFamilyId(), cardNo, cardStatus, cardType, pageIndex, pageSize));
    }

    /**
     * 申请提现
     *
     * @param amount
     * @param cardId
     * @return
     */
    @POST
    @Path("applyWithdraw")
    @Override
    public Result applyWithdraw(@FormParam("amount") BigDecimal amount, @FormParam("cardId") Long cardId) {

        Long familyId = this.getToken().getFamilyId();

        /*int dayOfWeek = DateUtil.dayOfWeek(new Date());
        if (dayOfWeek != 3 && dayOfWeek != 6) {
            return new Result(GlobalApiError.OperationFailed, "该功能仅周二、周五开放");
        }*/
        int dayOfWeek = LocalDate.now().getDayOfWeek().getValue();
        if (!Arrays.stream(withdrawDaysOfWeek).anyMatch(day -> day - dayOfWeek == 0)) {
            return new Result(GlobalApiError.OperationFailed, "该功能仅周 " + Arrays.toString(withdrawDaysOfWeek) + " 开放");
        }

        if (amount.compareTo(new BigDecimal(10000)) < 0) {
            return new Result(GlobalApiError.ServerError, "单次提现数量至少一万粉钻");
        }

        // 判断余额是否足够
        if (faFamilyWalletService.getBalance(familyId).compareTo(amount) == -1) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }

        Integer cardType = faFamilyBankcardService.getCardType(cardId);
        if (cardType == null) {
            return new Result(GlobalApiError.OperationFailed, "卡号不存在");
        }

        FamilyWalletWithdrawHisDO withdrawHisDO = new FamilyWalletWithdrawHisDO();
        withdrawHisDO.setHisId(identityService.getId());
        withdrawHisDO.setAmount(amount);
        withdrawHisDO.setApplyStatus(WalletWithdrawHisApplyStatus.WAIT.getValue());
        withdrawHisDO.setBankcardId(cardId);
        withdrawHisDO.setHisCreateTime(System.currentTimeMillis());
        withdrawHisDO.setFamilyId(familyId);
        withdrawHisDO.setUserTaxRatio(cardType == CardTypeEnum.PRIVATE.getValue() ? 0.08d : null);
        withdrawHisDO.setTaxAmount(withdrawHisDO.getUserTaxRatio() != null ? withdrawHisDO.getAmount().multiply(new BigDecimal(withdrawHisDO.getUserTaxRatio())).setScale(2, RoundingMode.UP) : null);
        withdrawHisDO.setActualAmount(amount.subtract(Optional.ofNullable(withdrawHisDO.getTaxAmount()).orElse(BigDecimal.ZERO).setScale(2, RoundingMode.DOWN)));
        withdrawHisDO.setFamilyAmount(withdrawHisDO.getActualAmount());
        withdrawHisDO.setWalletId(Convert.toStr(familyId));

        BigDecimal newBalance = faFamilyWalletWithdrawHisService.addWithdrawApply(withdrawHisDO, "提现转出");

        if (newBalance == null) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("balance", newBalance);
        return mResult.build();
    }

    /**
     * 获取余额
     *
     * @return
     */
    @GET
    @Path("getBalance")
    @Override
    public Result getBalance() {
        Long familyId = this.getToken().getFamilyId();
        MutableResult mResult = new MutableResult();
        mResult.put("familyId", familyId);
        mResult.put("balance", faFamilyWalletWithdrawHisService.getAmount(familyId));
        mResult.put("applyStatus", faFamilyWalletWithdrawHisService.getApplyStatus(familyId));
        return mResult.build();
    }

    /**
     * 获取家族提现申请列表
     *
     * @param applyStatus
     * @param startTime
     * @param endTime
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @Path("getFamilyWithdrawHisPageList")
    @Override
    public Result getFamilyWithdrawHisPageList(@QueryParam("applyStatus") Integer applyStatus,
                                               @QueryParam("startTime") Long startTime,
                                               @QueryParam("endTime") Long endTime,
                                               @QueryParam("pageIndex") int pageIndex,
                                               @QueryParam("pageSize") int pageSize) {
        return new Result(faFamilyWalletWithdrawHisService.getFamilyWithdrawHisPageList(this.getToken().getFamilyId(), applyStatus, startTime, endTime, pageIndex, pageSize));
    }

    /**
     * 获取账户明细列表
     *
     * @param lastId
     * @param isIncome
     * @param beginTime
     * @param endTime
     * @return
     */
    @GET
    @Path("getWalletHis")
    @Override
    public Result getWalletHis(@QueryParam("lastId") Long lastId, @QueryParam("isIncome") int isIncome, @QueryParam("beginTime") Long beginTime, @QueryParam("endTime") Long endTime) {
        long diffTime = endTime - beginTime;
        // 90天
        long ninetyDays = 90 * 24 * 3600 * 1000L;
        if (diffTime < 0 || diffTime > ninetyDays) {
            return new Result(GlobalApiError.ServerError, "时间跨度最长90天");
        }
        return new Result(faFamilyWalletService.getWalletHis(this.getToken().getFamilyId(), lastId, GlobalConfig.PAGE_SIZE, isIncome, beginTime, endTime));
    }

    /**
     * 求和
     *
     * @param isIncome
     * @param beginTime
     * @param endTime
     * @return
     */
    @GET
    @Path("getSumByTypeAndTime")
    @Override
    public Result getSumByTypeAndTime(@QueryParam("isIncome") int isIncome, @QueryParam("beginTime") Long beginTime, @QueryParam("endTime") Long endTime) {
        long diffTime = endTime - beginTime;
        // 90天
        long ninetyDays = 90 * 24 * 3600 * 1000L;
        if (diffTime < 0 || diffTime > ninetyDays) {
            return new Result(GlobalApiError.ServerError, "时间跨度最长90天");
        }
        return new Result(faFamilyWalletService.getSumByTypeAndTime(this.getToken().getFamilyId(), isIncome, beginTime, endTime));
    }
}
