package com.starbuds.server.api.family.v1.impl;

import com.starbuds.server.api.family.common.service.admin.FaRtcRoomService;
import com.starbuds.server.api.family.common.service.admin.FaStatRtcRoomService;
import com.starbuds.server.api.family.v1.facade.StatFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.statistics.StatRtcRoomTypeEnum;
import com.starbuds.server.common.pojo.api.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;

/**
 * 统计接口
 *
 * @author jiangxiaowei
 * @date 2020/11/22 19:22
 */
@Path("/v1/stat")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class StatFacadeImpl extends BaseFacade implements StatFacade {

    @Resource
    FaStatRtcRoomService faStatRtcRoomService;

    @Resource
    FaRtcRoomService faRtcRoomService;

    /**
     * 获取语音房间统计
     *
     * @param pageIndex
     * @param pageSize
     * @param roomId
     * @return
     */
    @GET
    @Path("getRoomStat")
    @Override
    public Result getRoomStat(@QueryParam("pageIndex") int pageIndex, @QueryParam("pageSize") int pageSize, @QueryParam("roomId") Long roomId) {
        if (roomId == null) {
            return new Result(GlobalApiError.BadRequest);
        }
        Long familyId = this.getToken().getFamilyId();
        if (!faRtcRoomService.hasPermission(familyId, roomId)) {
            return new Result(GlobalApiError.OperationFailed, "您没有该房间权限");
        }
        return new Result(faStatRtcRoomService.getRoomStat(pageIndex, pageSize, roomId, StatRtcRoomTypeEnum.SEND_STAR_DIAMOND.getValue()));
    }
}
